// KeyBoardKeys.cpp

#define _USE_MATH_DEFINES
#include <math.h>

#include "seeGLUT.h"

#include "LoadGame.h"  // because void KeyBoardKeys( unsigned char c ) assigns vaules to camera orientation and PAUSE flag
#include "physicalWorld.h" // sync Player with Camera
#include "KeyBoardKeys.h"

// used for looking up and down
float xrotrad, yrotrad;

// keep track of mouse pointer position
int lastx = -1, lasty = -1;

// to pause the simulation - it is used in void KeyBoardKeys( int c)
// and void UpdateTheWorld( float fDeltaTime )
bool bIsPaused = false;

// parses KeyBoard Input
void KeyBoardKeys( unsigned char c )
{
	// player walk direction from keyboard
	float xposWD = 0, yposWD = 0, zposWD = 0;

	switch( c )
	{
	case 'v':  // Look Down
		if (xrot < 90) xrot += 1;
		break;
	case 'f':  // Look Up
		if (xrot > -90) xrot -= 1;
		break;
	case 'w':  // Move Forward
		yrotrad = (yrot / 180 * (float)M_PI);
		xrotrad = (xrot / 180 * (float)M_PI); 
		xposWD += float(sin(yrotrad));
		zposWD -= float(cos(yrotrad));
		yposWD -= float(sin(xrotrad));
		break;
	case 's':  // Move Backwars
		yrotrad = (yrot / 180 * (float)M_PI);
		xrotrad = (xrot / 180 * (float)M_PI); 
		xposWD -= float(sin(yrotrad));
		zposWD += float(cos(yrotrad));
		yposWD += float(sin(xrotrad));
		break;
	case 'a':  // Move Left
		yrotrad = (yrot / 180 * (float)M_PI);
		xposWD -= float(cos(yrotrad)) * 0.5f;
		zposWD -= float(sin(yrotrad)) * 0.5f;
		break;
	case 'd':  // Move Right
		yrotrad = (yrot / 180 * (float)M_PI);
		xposWD += float(cos(yrotrad)) * 0.5f;
		zposWD += float(sin(yrotrad)) * 0.5f;
		break;
	case '.':  // Turn Right
		yrot += 1;
		if (yrot >360) yrot -= 360;
		break;
	case ',':  // Turn Left
		yrot -= 1;
		if (yrot < -360)yrot += 360;
		break;
	case ' ':  // Pause
		bIsPaused = !bIsPaused;
		break;
	case 27:  // press ESC to Quit
			glutLeaveMainLoop();
			break;
	}
	// (yposWD - 1) is for Gravity
	btVector3 walkDirection = btVector3( xposWD, (yposWD - 1), zposWD );
	keepMoving( walkDirection );
}

// parses mouse pointer positon
void MouseLook( int x, int y )
{
	if ( GLUT_ENTERED )
	{
		// Is this the First time called?
		if ( lastx == -1 && lasty == -1 )
			{
				lastx = x;
				lasty = y;
			}

		// Did the mouse pointer leave then come back to the GLUT window?
		if ( lastx == -2 && lasty == -2 )
			{
				lastx = x;
				lasty = y;
			}

		// change of position since last call
		int diffx=x-lastx;
		int diffy=y-lasty;
		// update current position for next call
		lastx=x;
		lasty=y;
		// update left and right [yrot] and up and down [xrot]
		// for void camera () used in void ThisIsTheWorld ()
		yrot += (float)diffx;
		// looking up and down are bounded by straight up and straight down
		if ( (xrot + (float)diffy) < 90 && (xrot + (float)diffy) > -90 ) xrot += (float)diffy;
	}
	else
	{
		lastx = -2;
		lasty = -2;
	}
	
}
