#include <cstdlib>
#include <iostream>
#include <ctime>

#include "seeGLUT.h"


#include "initTheWorld.h"
#include "ThisIsTheWorld.h"
#include "UpdateTheWorld.h"
#include "KeyBoardKeys.h"

// clock_t type to store the number of ticks between frames
std::clock_t g_PreviousTicks;
std::clock_t g_CurrentTicks;


// MAIN - MAIN - MAIN - - - //////////////////////

int main ( int argc, char* argv[] )
{
	std::cout << "I can't do that yet, Jim" << std::endl << std::endl;
	
	std::cout << "Forward: w" << std::endl;
	std::cout << "Backward: s" << std::endl;
	std::cout << "Strafe Left: a" << std::endl;
	std::cout << "Strafe Right: d" << std::endl;
	std::cout << "Turn Left: <" << std::endl;
	std::cout << "Turn Right: >" << std::endl;
	std::cout << "Look Up: f" << std::endl;
	std::cout << "Look Down: v" << std::endl;
	std::cout << "Pause: SPACEBAR" << std::endl;
	std::cout << "Quit: ESC" << std::endl << std::endl;

	initTheWorld();

	// Capture the previous time to calculate the delta time on the next frame
	// this is used in "void IdleGL()"
    g_PreviousTicks = std::clock();

	// create OpenGL context and render window
	initGLUT( argc, argv);

	// GLUT event processing loop does not stop
	// callback functions regitered with GLUT control the program
	// IdleGL method is invoked whenever No Event is being processed
	glutMainLoop();

	// destroy OpenGL context and render window
	Cleanup(0);
	
}

// Functions to handle GL events
void DisplayGL()
{
	// Clear the color buffer, and the depth buffer.
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// this is the World
	ThisIsTheWorld ();

	// flip the Back Buffer (offscreen rendering)
	// with the Front Buffer (onscreen display)
    glutSwapBuffers();

	// tells GLUT another frame is ready for rendering to the Back Buffer
	// then flags the current window to redisplay the Front Beffer
    glutPostRedisplay();
}

// updates the "behind the scenes" logic and workings of the program
void IdleGL()
{
	// Update our simulation
		// determine how much time has passed since IdleGL() was last called
		// std::clock() returns the number of clock ticks since the program started
    g_CurrentTicks = std::clock();
    float deltaTicks = ( (float)g_CurrentTicks - (float)g_PreviousTicks );
    g_PreviousTicks = g_CurrentTicks;

	// convert clock ticks to seconds
    float fDeltaTime = deltaTicks / (float)CLOCKS_PER_SEC;
	// CLOCKS_PER_SEC is defined in time.h

    // Give Action to the World
	UpdateTheWorld( fDeltaTime );
	

	// tells GLUT another frame is ready for rendering to the Back Buffer
	// then flags the current window to redisplay the Front Beffer
    glutPostRedisplay();
}

void KeyboardGL( unsigned char c, int x, int y )
{
	// parses KeyBoard Input
	KeyBoardKeys( c );

	// tells GLUT another frame is ready for rendering to the Back Buffer
	// then flags the current window to redisplay the Front Beffer
    glutPostRedisplay();
}

void MouseGL( int button, int state, int x, int y )
{
	// tells GLUT another frame is ready for rendering to the Back Buffer
	// then flags the current window to redisplay the Front Beffer
    glutPostRedisplay();
}

// passive mouse Function to detect mouse position on screen
// No Buttons Pressed
void MotionGL( int x, int y )
{
	// defined and declared in KeyBoardKeys.h and KeyBoardKeys.cpp
	MouseLook( x, y );
	// tells GLUT another frame is ready for rendering to the Back Buffer
	// then flags the current window to redisplay the Front Beffer
    glutPostRedisplay();
}

// passive mouse Function to detect mouse position on screen
// with One or more Buttons Pressed
void ButMotionGL( int x, int y )
{
	// tells GLUT another frame is ready for rendering to the Back Buffer
	// then flags the current window to redisplay the Front Beffer
    glutPostRedisplay();
}

// invoked by GLUT event loop
// whenever render window is resized and
// when render window is shown for the first time
void ReshapeGL( int w, int h )
{
	// Prevent A Divide By Zero error
    if ( h == 0)
    {
		// Making Height Equal One
        h = 1;
    }

    g_WindowWidth = w;
    g_WindowHeight = h;

	// set up viewport rectangle for the current OpenGL context
	// 0,0 is the top left corner and the width, height
	// specified fill the entire render window

    glViewport( 0, 0, g_WindowWidth, g_WindowHeight );

	// only put the projection matrices into the GL_PROJECTION matrix
	// The transforms that go from the model's space to camera space
	// should go into the GL_MODELVIEW matrix.
	
	// When you are looking at a scene, from GL point of view,
	// you have a camera and a lens.
	// ModelView is the matrix that represents your camera
	// (position, pointing, and up vector).
	// ProjectionView is the matrix that represents your camera's lens
	// (aperture, far-field, near-field, etc).

    // Setup the projection matrix
	// specify Matrix Mode to use and load it

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

	// Field of View, Aspect Ratio, Near and Far clipping plane

    gluPerspective( 60.0, (GLdouble)g_WindowWidth/(GLdouble)g_WindowHeight, 0.1, 100.0 );

	// sets flag that back buffer is ready to render
	// then sets flag that current window is ready to display from front buffer
    glutPostRedisplay();
}

