// physicalWorld.cpp

#include "LoadGame.h"  // because assigns a value to positiony[11]
#include "physicalWorld.h"
#include "ThisIsTheWorld.h"  // camera position (xpos, ypos, zpos)

// global variables that will be used by Bullet Physics World
// btBroadphaseInterface* broadphase;
btAxisSweep3* broadphase;
btDefaultCollisionConfiguration* collisionConfiguration;
btCollisionDispatcher* dispatcher;
btSequentialImpulseConstraintSolver* solver;
btDiscreteDynamicsWorld* dynamicsWorld;

// Player
btKinematicCharacterController* Player1;
btAxisSweep3* sweepBP;
btConvexShape* playerShape;
btPairCachingGhostObject* ghostObject;
btDefaultMotionState* playerMotionState;

const int maxProxies = 32766;
const int maxOverlap = 65535;

// global Shapes  and Bodies of the Bullet Physic sObjects
btCollisionShape* groundShape;
btCollisionShape* fallShape;

btRigidBody* groundRigidBody;
btRigidBody* fallRigidBody;

// Box Stuff
	btCollisionShape* boxShape[10];
	btRigidBody* boxRigidBody[10];
	btTransform boxTransform;
//	btScalar mass(0.);
	btVector3 localInertia(0,0,0);
	btDefaultMotionState* boxMotionState[10];


void initBullet()
{
	// eventually will replace loadGame() in initTheWorld.cpp
	initBPWorld();
	initBPObjects();
}

void destroyBullet()
{
	destroyBPObjects();
	destroyBPWorld();
}

// runs the simulations in the Bullet World
void upDateBullet()
{
	dynamicsWorld->stepSimulation(1 / 60.f, 10);
	btTransform trans;
	fallRigidBody->getMotionState()->getWorldTransform(trans);
	positionz[11] = trans.getOrigin().z();
	positiony[11] = trans.getOrigin().y();
	positionx[11] = trans.getOrigin().x();
	
	Player1->reset( dynamicsWorld );
}

// called by void camera () in ThisIsTheWorld.cpp
void keepLooking()
{
	zpos = Player1->getGhostObject()->getWorldTransform().getOrigin().z();
	ypos = Player1->getGhostObject()->getWorldTransform().getOrigin().y()+0.5f;
	xpos = Player1->getGhostObject()->getWorldTransform().getOrigin().x();
}

void keepMoving( btVector3 &walkDirection )
{
	Player1->setWalkDirection( walkDirection );
}

// initialize Bullet Physics World
void initBPWorld()
{
	///-----initialization_start-----
	// global variables that will be used by Bullet Physics World
	// are declared in main.cpp and defined Here

	///collision configuration contains default setup for memory, collision setup. Advanced users can create their own configuration.
	collisionConfiguration = new btDefaultCollisionConfiguration();

	///use the default collision dispatcher. For parallel processing you can use a diffent dispatcher (see Extras/BulletMultiThreaded)
	dispatcher = new btCollisionDispatcher(collisionConfiguration);

	btVector3 worldMin(-1000,-1000,-1000);
	btVector3 worldMax(1000,1000,1000);
	sweepBP = new btAxisSweep3(worldMin,worldMax);
	broadphase = sweepBP;

	///btDbvtBroadphase is a good general purpose broadphase. You can also try out btAxis3Sweep.
	///broadphase = new btDbvtBroadphase();

	///the default constraint solver. For parallel processing you can use a different solver (see Extras/BulletMultiThreaded)
	solver = new btSequentialImpulseConstraintSolver();

	dynamicsWorld = new btDiscreteDynamicsWorld(dispatcher, broadphase, solver, collisionConfiguration);
	broadphase->getOverlappingPairCache()->setInternalGhostPairCallback(new btGhostPairCallback());
	dynamicsWorld->setGravity(btVector3(0,-10,0));
	dynamicsWorld->getDispatchInfo().m_allowedCcdPenetration=0.0001f;

	///-----initialization_end-----
}


// clean up the Bullet Physics World
void destroyBPWorld()
{
	//cleanup in the reverse order of creation/initialization
	
	///-----cleanup_start-----
	
	//delete Bullet Physics world
	delete dynamicsWorld;
	delete solver;
	delete collisionConfiguration;
	delete dispatcher;
	delete sweepBP;

	///-----cleanup_end-----
}

// need for the Bullet Physics Objects

void initBPObjects()
{
	
	// create a static rigid body for the ground of the World
	groundShape = new btStaticPlaneShape(btVector3(0, 1, 0), 0);

	btDefaultMotionState* groundMotionState = new btDefaultMotionState(btTransform(btQuaternion(0, 0, 0, 1), btVector3(0, 0, 0)));
        btRigidBody::btRigidBodyConstructionInfo groundRigidBodyCI(0, groundMotionState, groundShape, btVector3(0, 0, 0));
        groundRigidBody = new btRigidBody(groundRigidBodyCI);
        dynamicsWorld->addRigidBody(groundRigidBody);

	// create a dynamic falling body
	fallShape = new btSphereShape(1);
	btDefaultMotionState* fallMotionState = new btDefaultMotionState(btTransform(btQuaternion(0, 0, 0, 1), btVector3( -40, 50, -50)));
        btScalar mass = 1;
        btVector3 fallInertia(0, 0, 0);
        fallShape->calculateLocalInertia(mass, fallInertia);
        btRigidBody::btRigidBodyConstructionInfo fallRigidBodyCI(mass, fallMotionState, fallShape, fallInertia);
        fallRigidBody = new btRigidBody(fallRigidBodyCI);
        dynamicsWorld->addRigidBody(fallRigidBody);

	// create ten boxes for the World
	for (int i=0;i<10;i++)
	{
		boxTransform.setIdentity();
		boxTransform.setOrigin(btVector3(-positionx[i + 1] * 10, 0, -positionz[i + 1] * 10));
		
		boxShape[i] = new btBoxShape(btVector3( 2, 2, 2 ));
		boxShape[i]->calculateLocalInertia(mass,localInertia);

		//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
		boxMotionState[i] = new btDefaultMotionState(boxTransform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,boxMotionState[i],boxShape[i],localInertia);
		boxRigidBody[i] = new btRigidBody(rbInfo);
	
		//add the body to the dynamics world
		dynamicsWorld->addRigidBody(boxRigidBody[i]);
	}
	
	// add the Player to the Wotld
		playerShape = new btCapsuleShape(0.25, 1);
		ghostObject = new btPairCachingGhostObject();
		ghostObject->setWorldTransform(btTransform(btQuaternion(0,0,0,1),btVector3(-20,0,-20)));
		dynamicsWorld->getPairCache()->setInternalGhostPairCallback(new btGhostPairCallback());
		ghostObject->setCollisionShape(playerShape);
		ghostObject->setCollisionFlags(btCollisionObject::CF_CHARACTER_OBJECT);
		Player1 = new btKinematicCharacterController(ghostObject,playerShape,0.5);
		dynamicsWorld->addCollisionObject(ghostObject,btBroadphaseProxy::CharacterFilter, btBroadphaseProxy::StaticFilter|btBroadphaseProxy::DefaultFilter);
		dynamicsWorld->addAction(Player1);

		zpos = Player1->getGhostObject()->getWorldTransform().getOrigin().z();
		ypos = Player1->getGhostObject()->getWorldTransform().getOrigin().y()+0.5f;
		xpos = Player1->getGhostObject()->getWorldTransform().getOrigin().x();

}

void destroyBPObjects()
{
	//cleanup in the reverse order of creation/initialization
	
	///-----cleanup_start-----

	//cleanup player in rrverse order
	dynamicsWorld->removeAction(Player1);
	dynamicsWorld->removeCollisionObject(ghostObject);
	delete Player1;
	delete ghostObject;
	delete playerShape;

	// delete the ten boxes
	for (int i=0;i<10;i++)
	{
		dynamicsWorld->removeRigidBody(boxRigidBody[i]);
		delete boxRigidBody[i]->getMotionState();
		delete boxRigidBody[i];
		delete boxShape[i];
	}

	// delete the falling body
	dynamicsWorld->removeRigidBody(fallRigidBody);
	delete fallRigidBody->getMotionState();
	delete fallRigidBody;

	// delete the ground of the World
	dynamicsWorld->removeRigidBody(groundRigidBody);
	delete groundRigidBody->getMotionState();
	delete groundRigidBody;

	// delete collision shapes
	delete fallShape;
	delete groundShape;

	///-----cleanup_end-----
}

